from django.utils.translation import ugettext as _
from django.template import Library

register = Library()

def _get_default_letters(model_admin=None):
    from django.conf import settings
    import string
    default_letters = getattr(settings, 'DEFAULT_ALPHABET', string.digits + string.ascii_uppercase)
    if model_admin and hasattr(model_admin, 'DEFAULT_ALPHABET'):
        default_letters = model_admin.DEFAULT_ALPHABET
    if callable(default_letters):
        return set(default_letters())
    elif isinstance(default_letters, unicode):
        return set([x for x in default_letters])
    elif isinstance(default_letters, str):
        return set([x for x in default_letters.decode('utf8')])
    elif isinstance(default_letters, (tuple, list)):
        return set(default_letters)


def _get_available_letters(field_name, db_table):
    from django.db import connection, transaction
    from django.conf import settings
    qn = connection.ops.quote_name
    sql = "SELECT DISTINCT UPPER(SUBSTR(%s, 1, 1)) as letter FROM %s" \
                % (qn(field_name), qn(db_table))
    cursor = connection.cursor()
    cursor.execute(sql)
    rows = cursor.fetchall() or ()
    return set([row[0] for row in rows if row[0] is not None])


def alphabet(cl):
    if not getattr(cl.model_admin, 'alphabet_filter', False):
        return
    field_name = cl.model_admin.alphabet_filter
    alpha_field = '%s__istartswith' % field_name
    alpha_lookup = cl.params.get(alpha_field, '')
    link = lambda d: cl.get_query_string(d)
    
    letters_used = _get_available_letters(field_name, cl.model._meta.db_table)
    all_letters = list(_get_default_letters(cl.model_admin) | letters_used)
    all_letters.sort()
    
    choices = [{
        'link': link({alpha_field: letter}), 
        'title': letter,
        'active': letter == alpha_lookup,
        'has_entries': letter in letters_used,} for letter in all_letters]
    all_letters = [{
        'link': cl.get_query_string(None,alpha_field),
        'title': _('All'),
        'active': '' == alpha_lookup,
        'has_entries': True
    },]
    return {'choices': all_letters + choices}
alphabet = register.inclusion_tag('admin/alphabet.html')(alphabet)




from django.db import models

# Create your models here.

class TestName(models.Model):
    """A testing module"""
    name = models.CharField(max_length=255)
    sorted_name = models.CharField(max_length=255)

    class Meta:
        pass

    def __unicode__(self):
        return self.name



import os, sys
from setuptools import setup, find_packages

def read_file(filename):
    path = os.path.abspath(os.path.dirname(__file__))
    filepath = os.path.join(path, filename)
    try:
        return open(filepath).read()
    except:
        return ''

setup(
    name = "django-alphafilter",
    version = __import__('alphafilter').get_version().replace(' ', '-'),
    url = 'http://github.com/coordt/django-alphabetfilter',
    author = 'Corey Oordt',
    author_email = 'coreyoordt@gmail.com',
    description = 'An alphabetical filter for Django\'s admin that works like date_hierarchy',
    long_description = read_file('README'),
    packages = find_packages(),
    license = 'Apache 2.0',
    include_package_data = True,
    install_requires=read_file('requirements.txt'),
    classifiers = [
    ],
)



from django.contrib import admin
from models import TestName

class TestNameAdmin(admin.ModelAdmin):
    model = TestName
    alphabet_filter = 'sorted_name'
    
    ## Testing a custom Default Alphabet
    #DEFAULT_ALPHABET = 'ABC'
    
    ## Testing a blank alphabet-- only shows the characters in the database
    #DEFAULT_ALPHABET = ''
    
    ## Testing a callable
    def DEFAULT_ALPHABET(self):
        return "I D K W"

admin.site.register(TestName, TestNameAdmin)